% A script that solves the linear complementarity problem associated with 
% the contact mechanics problem
%
% h = K*p+g,  h'*p=0, h,p >=0
%
% where h is the gap between two elastic (half-space) bodies and p is the 
% corresponding contact pressure.
%
% The problem is stated in a Hertzian dimensionless formulation
%
% x_dl = x/b,   h_dl = h/(b^2/Rx),   p_dl = p/ph,   w_dl  = w/(ph*b).
% 
% b  = sqrt(8*w*Rx/(pi*E)),   ph = w/(w_dl*b) = sqrt(w*E/(2*pi*Rx)),   
% 
% where
%
% 2/E' = (1-nu1^2)/E1+(1-nu2^2)/E2,   1/Rx = 1/R1 + 1/R2,
%
% % Copyright (c) 2013 Andreas Almqvist
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are 
% met: 
% 
% 1. Redistributions of source code must retain the above copyright notice,
%    this list of conditions and the following disclaimer. 
% 2. Redistributions in binary form must reproduce the above copyright 
%    notice, this list of conditions and the following disclaimer in the
%    documentation and/or other materials provided with the distribution. 
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS 
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
clear all;

%% Input
% Physical
R1 = 0.05;  % Curvature of upper body
R2 = 0.03;  % Curvature of lower body
E1 = 210e3; nu1 = 0.3; % E-mod and Poissons ratios of upper body
E2 = 210e3; nu2 = 0.3; % E-mod and Poissons ratios of lower body
iv = [-0.5,0.5]*1e-3;  % Computational domain

% Solver spec.
N  = 2^7+1;       % Number of nodes
 
%% Preprocessing
Rx = 1/(1/R1 + 1/R2);               % Reduced radius of curvature
E  = 2/((1-nu1^2)/E1+(1-nu2^2)/E2); % Effective elastic modulus
% Domain
dx = ((iv(2)-iv(1)))/(N-1);     % Meshsize
x  = iv(1):dx:iv(2);            % Discrete domain

% Matrix kernel 
j  = 1:N;
d1  = (1-j+0.5)*dx; d2=(1-j-0.5)*dx;
k  = d1.*( log( abs( d1 ) )  - 1) - d2.*( log(abs( d2 ) ) - 1);
K  = 4*toeplitz(-k)/(pi*E);

% Define the initial gap between the surfaces
g0 = -4e-6;             % Rigid body separation
gs = (x.^2/(2*Rx)).';   % Shape of undeformed surface
g  = g0 + gs;           % Initial gap 

%% Solver
% Call the pivoting algorithm
tic; [h,p,retcode] = LCPSolve(K,g,1e-15); toc; 

%% Post processing and visualisation
w  = sum(p(1:N-1)+p((2:N)))*dx/2; % The contact load

ph = sqrt(w*E/(2*pi*Rx));   % Hertzian pressure
b  = sqrt(8*w*Rx/(pi*E));   % Hertzian contact half width

x_dl  = x/b;                % Dimensionless domain
g_dl  = g/(b^2/Rx);         % Shape of undeformed surface
h_dl  = h/(b^2/Rx);         % Shape of deformed surface
p_dl  = p/ph;               % Pressure distribution

% Plotting dimensionless quantities
figure(3); clf;
plot(x_dl,g_dl.*(g_dl>=0),'b')
hold on;
plot(x_dl,h_dl,'r')
plot(x_dl,p_dl,'k.-')
grid on;
legend('Initial gap','deformed gap','pressure')